# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .chat import (
    ChatResource,
    AsyncChatResource,
    ChatResourceWithRawResponse,
    AsyncChatResourceWithRawResponse,
    ChatResourceWithStreamingResponse,
    AsyncChatResourceWithStreamingResponse,
)
from .audio import (
    AudioResource,
    AsyncAudioResource,
    AudioResourceWithRawResponse,
    AsyncAudioResourceWithRawResponse,
    AudioResourceWithStreamingResponse,
    AsyncAudioResourceWithStreamingResponse,
)
from ...types import ai_summarize_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .clusters import (
    ClustersResource,
    AsyncClustersResource,
    ClustersResourceWithRawResponse,
    AsyncClustersResourceWithRawResponse,
    ClustersResourceWithStreamingResponse,
    AsyncClustersResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .mcp_servers import (
    McpServersResource,
    AsyncMcpServersResource,
    McpServersResourceWithRawResponse,
    AsyncMcpServersResourceWithRawResponse,
    McpServersResourceWithStreamingResponse,
    AsyncMcpServersResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from .assistants.assistants import (
    AssistantsResource,
    AsyncAssistantsResource,
    AssistantsResourceWithRawResponse,
    AsyncAssistantsResourceWithRawResponse,
    AssistantsResourceWithStreamingResponse,
    AsyncAssistantsResourceWithStreamingResponse,
)
from .embeddings.embeddings import (
    EmbeddingsResource,
    AsyncEmbeddingsResource,
    EmbeddingsResourceWithRawResponse,
    AsyncEmbeddingsResourceWithRawResponse,
    EmbeddingsResourceWithStreamingResponse,
    AsyncEmbeddingsResourceWithStreamingResponse,
)
from .fine_tuning.fine_tuning import (
    FineTuningResource,
    AsyncFineTuningResource,
    FineTuningResourceWithRawResponse,
    AsyncFineTuningResourceWithRawResponse,
    FineTuningResourceWithStreamingResponse,
    AsyncFineTuningResourceWithStreamingResponse,
)
from .integrations.integrations import (
    IntegrationsResource,
    AsyncIntegrationsResource,
    IntegrationsResourceWithRawResponse,
    AsyncIntegrationsResourceWithRawResponse,
    IntegrationsResourceWithStreamingResponse,
    AsyncIntegrationsResourceWithStreamingResponse,
)
from .conversations.conversations import (
    ConversationsResource,
    AsyncConversationsResource,
    ConversationsResourceWithRawResponse,
    AsyncConversationsResourceWithRawResponse,
    ConversationsResourceWithStreamingResponse,
    AsyncConversationsResourceWithStreamingResponse,
)
from ...types.ai_summarize_response import AISummarizeResponse
from ...types.ai_retrieve_models_response import AIRetrieveModelsResponse

__all__ = ["AIResource", "AsyncAIResource"]


class AIResource(SyncAPIResource):
    @cached_property
    def assistants(self) -> AssistantsResource:
        return AssistantsResource(self._client)

    @cached_property
    def audio(self) -> AudioResource:
        return AudioResource(self._client)

    @cached_property
    def chat(self) -> ChatResource:
        return ChatResource(self._client)

    @cached_property
    def clusters(self) -> ClustersResource:
        return ClustersResource(self._client)

    @cached_property
    def conversations(self) -> ConversationsResource:
        return ConversationsResource(self._client)

    @cached_property
    def embeddings(self) -> EmbeddingsResource:
        return EmbeddingsResource(self._client)

    @cached_property
    def fine_tuning(self) -> FineTuningResource:
        return FineTuningResource(self._client)

    @cached_property
    def integrations(self) -> IntegrationsResource:
        return IntegrationsResource(self._client)

    @cached_property
    def mcp_servers(self) -> McpServersResource:
        return McpServersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AIResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AIResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AIResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AIResourceWithStreamingResponse(self)

    def retrieve_models(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AIRetrieveModelsResponse:
        """
        This endpoint returns a list of Open Source and OpenAI models that are available
        for use. <br /><br /> **Note**: Model `id`'s will be in the form
        `{source}/{model_name}`. For example `openai/gpt-4` or
        `mistralai/Mistral-7B-Instruct-v0.1` consistent with HuggingFace naming
        conventions.
        """
        return self._get(
            "/ai/models",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AIRetrieveModelsResponse,
        )

    def summarize(
        self,
        *,
        bucket: str,
        filename: str,
        system_prompt: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AISummarizeResponse:
        """
        Generate a summary of a file's contents.

        Supports the following text formats:

        - PDF, HTML, txt, json, csv

        Supports the following media formats (billed for both the transcription and
        summary):

        - flac, mp3, mp4, mpeg, mpga, m4a, ogg, wav, or webm
        - Up to 100 MB

        Args:
          bucket: The name of the bucket that contains the file to be summarized.

          filename: The name of the file to be summarized.

          system_prompt: A system prompt to guide the summary generation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ai/summarize",
            body=maybe_transform(
                {
                    "bucket": bucket,
                    "filename": filename,
                    "system_prompt": system_prompt,
                },
                ai_summarize_params.AISummarizeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AISummarizeResponse,
        )


class AsyncAIResource(AsyncAPIResource):
    @cached_property
    def assistants(self) -> AsyncAssistantsResource:
        return AsyncAssistantsResource(self._client)

    @cached_property
    def audio(self) -> AsyncAudioResource:
        return AsyncAudioResource(self._client)

    @cached_property
    def chat(self) -> AsyncChatResource:
        return AsyncChatResource(self._client)

    @cached_property
    def clusters(self) -> AsyncClustersResource:
        return AsyncClustersResource(self._client)

    @cached_property
    def conversations(self) -> AsyncConversationsResource:
        return AsyncConversationsResource(self._client)

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResource:
        return AsyncEmbeddingsResource(self._client)

    @cached_property
    def fine_tuning(self) -> AsyncFineTuningResource:
        return AsyncFineTuningResource(self._client)

    @cached_property
    def integrations(self) -> AsyncIntegrationsResource:
        return AsyncIntegrationsResource(self._client)

    @cached_property
    def mcp_servers(self) -> AsyncMcpServersResource:
        return AsyncMcpServersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAIResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAIResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAIResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAIResourceWithStreamingResponse(self)

    async def retrieve_models(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AIRetrieveModelsResponse:
        """
        This endpoint returns a list of Open Source and OpenAI models that are available
        for use. <br /><br /> **Note**: Model `id`'s will be in the form
        `{source}/{model_name}`. For example `openai/gpt-4` or
        `mistralai/Mistral-7B-Instruct-v0.1` consistent with HuggingFace naming
        conventions.
        """
        return await self._get(
            "/ai/models",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AIRetrieveModelsResponse,
        )

    async def summarize(
        self,
        *,
        bucket: str,
        filename: str,
        system_prompt: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AISummarizeResponse:
        """
        Generate a summary of a file's contents.

        Supports the following text formats:

        - PDF, HTML, txt, json, csv

        Supports the following media formats (billed for both the transcription and
        summary):

        - flac, mp3, mp4, mpeg, mpga, m4a, ogg, wav, or webm
        - Up to 100 MB

        Args:
          bucket: The name of the bucket that contains the file to be summarized.

          filename: The name of the file to be summarized.

          system_prompt: A system prompt to guide the summary generation.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ai/summarize",
            body=await async_maybe_transform(
                {
                    "bucket": bucket,
                    "filename": filename,
                    "system_prompt": system_prompt,
                },
                ai_summarize_params.AISummarizeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AISummarizeResponse,
        )


class AIResourceWithRawResponse:
    def __init__(self, ai: AIResource) -> None:
        self._ai = ai

        self.retrieve_models = to_raw_response_wrapper(
            ai.retrieve_models,
        )
        self.summarize = to_raw_response_wrapper(
            ai.summarize,
        )

    @cached_property
    def assistants(self) -> AssistantsResourceWithRawResponse:
        return AssistantsResourceWithRawResponse(self._ai.assistants)

    @cached_property
    def audio(self) -> AudioResourceWithRawResponse:
        return AudioResourceWithRawResponse(self._ai.audio)

    @cached_property
    def chat(self) -> ChatResourceWithRawResponse:
        return ChatResourceWithRawResponse(self._ai.chat)

    @cached_property
    def clusters(self) -> ClustersResourceWithRawResponse:
        return ClustersResourceWithRawResponse(self._ai.clusters)

    @cached_property
    def conversations(self) -> ConversationsResourceWithRawResponse:
        return ConversationsResourceWithRawResponse(self._ai.conversations)

    @cached_property
    def embeddings(self) -> EmbeddingsResourceWithRawResponse:
        return EmbeddingsResourceWithRawResponse(self._ai.embeddings)

    @cached_property
    def fine_tuning(self) -> FineTuningResourceWithRawResponse:
        return FineTuningResourceWithRawResponse(self._ai.fine_tuning)

    @cached_property
    def integrations(self) -> IntegrationsResourceWithRawResponse:
        return IntegrationsResourceWithRawResponse(self._ai.integrations)

    @cached_property
    def mcp_servers(self) -> McpServersResourceWithRawResponse:
        return McpServersResourceWithRawResponse(self._ai.mcp_servers)


class AsyncAIResourceWithRawResponse:
    def __init__(self, ai: AsyncAIResource) -> None:
        self._ai = ai

        self.retrieve_models = async_to_raw_response_wrapper(
            ai.retrieve_models,
        )
        self.summarize = async_to_raw_response_wrapper(
            ai.summarize,
        )

    @cached_property
    def assistants(self) -> AsyncAssistantsResourceWithRawResponse:
        return AsyncAssistantsResourceWithRawResponse(self._ai.assistants)

    @cached_property
    def audio(self) -> AsyncAudioResourceWithRawResponse:
        return AsyncAudioResourceWithRawResponse(self._ai.audio)

    @cached_property
    def chat(self) -> AsyncChatResourceWithRawResponse:
        return AsyncChatResourceWithRawResponse(self._ai.chat)

    @cached_property
    def clusters(self) -> AsyncClustersResourceWithRawResponse:
        return AsyncClustersResourceWithRawResponse(self._ai.clusters)

    @cached_property
    def conversations(self) -> AsyncConversationsResourceWithRawResponse:
        return AsyncConversationsResourceWithRawResponse(self._ai.conversations)

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResourceWithRawResponse:
        return AsyncEmbeddingsResourceWithRawResponse(self._ai.embeddings)

    @cached_property
    def fine_tuning(self) -> AsyncFineTuningResourceWithRawResponse:
        return AsyncFineTuningResourceWithRawResponse(self._ai.fine_tuning)

    @cached_property
    def integrations(self) -> AsyncIntegrationsResourceWithRawResponse:
        return AsyncIntegrationsResourceWithRawResponse(self._ai.integrations)

    @cached_property
    def mcp_servers(self) -> AsyncMcpServersResourceWithRawResponse:
        return AsyncMcpServersResourceWithRawResponse(self._ai.mcp_servers)


class AIResourceWithStreamingResponse:
    def __init__(self, ai: AIResource) -> None:
        self._ai = ai

        self.retrieve_models = to_streamed_response_wrapper(
            ai.retrieve_models,
        )
        self.summarize = to_streamed_response_wrapper(
            ai.summarize,
        )

    @cached_property
    def assistants(self) -> AssistantsResourceWithStreamingResponse:
        return AssistantsResourceWithStreamingResponse(self._ai.assistants)

    @cached_property
    def audio(self) -> AudioResourceWithStreamingResponse:
        return AudioResourceWithStreamingResponse(self._ai.audio)

    @cached_property
    def chat(self) -> ChatResourceWithStreamingResponse:
        return ChatResourceWithStreamingResponse(self._ai.chat)

    @cached_property
    def clusters(self) -> ClustersResourceWithStreamingResponse:
        return ClustersResourceWithStreamingResponse(self._ai.clusters)

    @cached_property
    def conversations(self) -> ConversationsResourceWithStreamingResponse:
        return ConversationsResourceWithStreamingResponse(self._ai.conversations)

    @cached_property
    def embeddings(self) -> EmbeddingsResourceWithStreamingResponse:
        return EmbeddingsResourceWithStreamingResponse(self._ai.embeddings)

    @cached_property
    def fine_tuning(self) -> FineTuningResourceWithStreamingResponse:
        return FineTuningResourceWithStreamingResponse(self._ai.fine_tuning)

    @cached_property
    def integrations(self) -> IntegrationsResourceWithStreamingResponse:
        return IntegrationsResourceWithStreamingResponse(self._ai.integrations)

    @cached_property
    def mcp_servers(self) -> McpServersResourceWithStreamingResponse:
        return McpServersResourceWithStreamingResponse(self._ai.mcp_servers)


class AsyncAIResourceWithStreamingResponse:
    def __init__(self, ai: AsyncAIResource) -> None:
        self._ai = ai

        self.retrieve_models = async_to_streamed_response_wrapper(
            ai.retrieve_models,
        )
        self.summarize = async_to_streamed_response_wrapper(
            ai.summarize,
        )

    @cached_property
    def assistants(self) -> AsyncAssistantsResourceWithStreamingResponse:
        return AsyncAssistantsResourceWithStreamingResponse(self._ai.assistants)

    @cached_property
    def audio(self) -> AsyncAudioResourceWithStreamingResponse:
        return AsyncAudioResourceWithStreamingResponse(self._ai.audio)

    @cached_property
    def chat(self) -> AsyncChatResourceWithStreamingResponse:
        return AsyncChatResourceWithStreamingResponse(self._ai.chat)

    @cached_property
    def clusters(self) -> AsyncClustersResourceWithStreamingResponse:
        return AsyncClustersResourceWithStreamingResponse(self._ai.clusters)

    @cached_property
    def conversations(self) -> AsyncConversationsResourceWithStreamingResponse:
        return AsyncConversationsResourceWithStreamingResponse(self._ai.conversations)

    @cached_property
    def embeddings(self) -> AsyncEmbeddingsResourceWithStreamingResponse:
        return AsyncEmbeddingsResourceWithStreamingResponse(self._ai.embeddings)

    @cached_property
    def fine_tuning(self) -> AsyncFineTuningResourceWithStreamingResponse:
        return AsyncFineTuningResourceWithStreamingResponse(self._ai.fine_tuning)

    @cached_property
    def integrations(self) -> AsyncIntegrationsResourceWithStreamingResponse:
        return AsyncIntegrationsResourceWithStreamingResponse(self._ai.integrations)

    @cached_property
    def mcp_servers(self) -> AsyncMcpServersResourceWithStreamingResponse:
        return AsyncMcpServersResourceWithStreamingResponse(self._ai.mcp_servers)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .hangup_tool_param import HangupToolParam
from .webhook_tool_param import WebhookToolParam
from .transfer_tool_param import TransferToolParam
from .retrieval_tool_param import RetrievalToolParam

__all__ = [
    "AssistantParam",
    "Tool",
    "ToolBookAppointmentTool",
    "ToolBookAppointmentToolBookAppointment",
    "ToolCheckAvailabilityTool",
    "ToolCheckAvailabilityToolCheckAvailability",
]


class ToolBookAppointmentToolBookAppointment(TypedDict, total=False):
    api_key_ref: Required[str]
    """Reference to an integration secret that contains your Cal.com API key.

    You would pass the `identifier` for an integration secret
    [/v2/integration_secrets](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
    that refers to your Cal.com API key.
    """

    event_type_id: Required[int]
    """Event Type ID for which slots are being fetched.

    [cal.com](https://cal.com/docs/api-reference/v2/bookings/create-a-booking#body-event-type-id)
    """

    attendee_name: str
    """
    The name of the attendee
    [cal.com](https://cal.com/docs/api-reference/v2/bookings/create-a-booking#body-attendee-name).
    If not provided, the assistant will ask for the attendee's name.
    """

    attendee_timezone: str
    """
    The timezone of the attendee
    [cal.com](https://cal.com/docs/api-reference/v2/bookings/create-a-booking#body-attendee-timezone).
    If not provided, the assistant will ask for the attendee's timezone.
    """


class ToolBookAppointmentTool(TypedDict, total=False):
    book_appointment: Required[ToolBookAppointmentToolBookAppointment]

    type: Required[Literal["book_appointment"]]


class ToolCheckAvailabilityToolCheckAvailability(TypedDict, total=False):
    api_key_ref: Required[str]
    """Reference to an integration secret that contains your Cal.com API key.

    You would pass the `identifier` for an integration secret
    [/v2/integration_secrets](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
    that refers to your Cal.com API key.
    """

    event_type_id: Required[int]
    """Event Type ID for which slots are being fetched.

    [cal.com](https://cal.com/docs/api-reference/v2/slots/get-available-slots#parameter-event-type-id)
    """


class ToolCheckAvailabilityTool(TypedDict, total=False):
    check_availability: Required[ToolCheckAvailabilityToolCheckAvailability]

    type: Required[Literal["check_availability"]]


Tool: TypeAlias = Union[
    ToolBookAppointmentTool,
    ToolCheckAvailabilityTool,
    WebhookToolParam,
    HangupToolParam,
    TransferToolParam,
    RetrievalToolParam,
]


class AssistantParam(TypedDict, total=False):
    """
    Assistant configuration including choice of LLM, custom instructions, and tools.
    """

    instructions: str
    """The system instructions that the voice assistant uses during the gather command"""

    model: str
    """The model to be used by the voice assistant."""

    openai_api_key_ref: str
    """This is necessary only if the model selected is from OpenAI.

    You would pass the `identifier` for an integration secret
    [/v2/integration_secrets](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
    that refers to your OpenAI API Key. Warning: Free plans are unlikely to work
    with this integration.
    """

    tools: Iterable[Tool]
    """The tools that the voice assistant can use."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["GlobalIPAllowedPortListResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    first_port: Optional[int] = None
    """First port of a range."""

    last_port: Optional[int] = None
    """Last port of a range."""

    name: Optional[str] = None
    """A name for the Global IP ports range."""

    protocol_code: Optional[str] = None
    """The Global IP Protocol code."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""


class GlobalIPAllowedPortListResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["AnchorsiteOverride"]

AnchorsiteOverride: TypeAlias = Literal[
    "Latency",
    "Chicago, IL",
    "Ashburn, VA",
    "San Jose, CA",
    "Sydney, Australia",
    "Amsterdam, Netherlands",
    "London, UK",
    "Toronto, Canada",
    "Vancouver, Canada",
    "Frankfurt, Germany",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.rooms.sessions import action_kick_params, action_mute_params, action_unmute_params
from ....types.rooms.sessions.action_end_response import ActionEndResponse
from ....types.rooms.sessions.action_kick_response import ActionKickResponse
from ....types.rooms.sessions.action_mute_response import ActionMuteResponse
from ....types.rooms.sessions.action_unmute_response import ActionUnmuteResponse

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)

    def end(
        self,
        room_session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionEndResponse:
        """
        Note: this will also kick all participants currently present in the room

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_session_id:
            raise ValueError(f"Expected a non-empty value for `room_session_id` but received {room_session_id!r}")
        return self._post(
            f"/room_sessions/{room_session_id}/actions/end",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionEndResponse,
        )

    def kick(
        self,
        room_session_id: str,
        *,
        exclude: SequenceNotStr[str] | Omit = omit,
        participants: Union[Literal["all"], SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionKickResponse:
        """
        Kick participants from a room session.

        Args:
          exclude: List of participant id to exclude from the action.

          participants: Either a list of participant id to perform the action on, or the keyword "all"
              to perform the action on all participant.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_session_id:
            raise ValueError(f"Expected a non-empty value for `room_session_id` but received {room_session_id!r}")
        return self._post(
            f"/room_sessions/{room_session_id}/actions/kick",
            body=maybe_transform(
                {
                    "exclude": exclude,
                    "participants": participants,
                },
                action_kick_params.ActionKickParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionKickResponse,
        )

    def mute(
        self,
        room_session_id: str,
        *,
        exclude: SequenceNotStr[str] | Omit = omit,
        participants: Union[Literal["all"], SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionMuteResponse:
        """
        Mute participants in room session.

        Args:
          exclude: List of participant id to exclude from the action.

          participants: Either a list of participant id to perform the action on, or the keyword "all"
              to perform the action on all participant.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_session_id:
            raise ValueError(f"Expected a non-empty value for `room_session_id` but received {room_session_id!r}")
        return self._post(
            f"/room_sessions/{room_session_id}/actions/mute",
            body=maybe_transform(
                {
                    "exclude": exclude,
                    "participants": participants,
                },
                action_mute_params.ActionMuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionMuteResponse,
        )

    def unmute(
        self,
        room_session_id: str,
        *,
        exclude: SequenceNotStr[str] | Omit = omit,
        participants: Union[Literal["all"], SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionUnmuteResponse:
        """
        Unmute participants in room session.

        Args:
          exclude: List of participant id to exclude from the action.

          participants: Either a list of participant id to perform the action on, or the keyword "all"
              to perform the action on all participant.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_session_id:
            raise ValueError(f"Expected a non-empty value for `room_session_id` but received {room_session_id!r}")
        return self._post(
            f"/room_sessions/{room_session_id}/actions/unmute",
            body=maybe_transform(
                {
                    "exclude": exclude,
                    "participants": participants,
                },
                action_unmute_params.ActionUnmuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionUnmuteResponse,
        )


class AsyncActionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)

    async def end(
        self,
        room_session_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionEndResponse:
        """
        Note: this will also kick all participants currently present in the room

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_session_id:
            raise ValueError(f"Expected a non-empty value for `room_session_id` but received {room_session_id!r}")
        return await self._post(
            f"/room_sessions/{room_session_id}/actions/end",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionEndResponse,
        )

    async def kick(
        self,
        room_session_id: str,
        *,
        exclude: SequenceNotStr[str] | Omit = omit,
        participants: Union[Literal["all"], SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionKickResponse:
        """
        Kick participants from a room session.

        Args:
          exclude: List of participant id to exclude from the action.

          participants: Either a list of participant id to perform the action on, or the keyword "all"
              to perform the action on all participant.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_session_id:
            raise ValueError(f"Expected a non-empty value for `room_session_id` but received {room_session_id!r}")
        return await self._post(
            f"/room_sessions/{room_session_id}/actions/kick",
            body=await async_maybe_transform(
                {
                    "exclude": exclude,
                    "participants": participants,
                },
                action_kick_params.ActionKickParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionKickResponse,
        )

    async def mute(
        self,
        room_session_id: str,
        *,
        exclude: SequenceNotStr[str] | Omit = omit,
        participants: Union[Literal["all"], SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionMuteResponse:
        """
        Mute participants in room session.

        Args:
          exclude: List of participant id to exclude from the action.

          participants: Either a list of participant id to perform the action on, or the keyword "all"
              to perform the action on all participant.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_session_id:
            raise ValueError(f"Expected a non-empty value for `room_session_id` but received {room_session_id!r}")
        return await self._post(
            f"/room_sessions/{room_session_id}/actions/mute",
            body=await async_maybe_transform(
                {
                    "exclude": exclude,
                    "participants": participants,
                },
                action_mute_params.ActionMuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionMuteResponse,
        )

    async def unmute(
        self,
        room_session_id: str,
        *,
        exclude: SequenceNotStr[str] | Omit = omit,
        participants: Union[Literal["all"], SequenceNotStr[str]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionUnmuteResponse:
        """
        Unmute participants in room session.

        Args:
          exclude: List of participant id to exclude from the action.

          participants: Either a list of participant id to perform the action on, or the keyword "all"
              to perform the action on all participant.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_session_id:
            raise ValueError(f"Expected a non-empty value for `room_session_id` but received {room_session_id!r}")
        return await self._post(
            f"/room_sessions/{room_session_id}/actions/unmute",
            body=await async_maybe_transform(
                {
                    "exclude": exclude,
                    "participants": participants,
                },
                action_unmute_params.ActionUnmuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionUnmuteResponse,
        )


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.end = to_raw_response_wrapper(
            actions.end,
        )
        self.kick = to_raw_response_wrapper(
            actions.kick,
        )
        self.mute = to_raw_response_wrapper(
            actions.mute,
        )
        self.unmute = to_raw_response_wrapper(
            actions.unmute,
        )


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.end = async_to_raw_response_wrapper(
            actions.end,
        )
        self.kick = async_to_raw_response_wrapper(
            actions.kick,
        )
        self.mute = async_to_raw_response_wrapper(
            actions.mute,
        )
        self.unmute = async_to_raw_response_wrapper(
            actions.unmute,
        )


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.end = to_streamed_response_wrapper(
            actions.end,
        )
        self.kick = to_streamed_response_wrapper(
            actions.kick,
        )
        self.mute = to_streamed_response_wrapper(
            actions.mute,
        )
        self.unmute = to_streamed_response_wrapper(
            actions.unmute,
        )


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.end = async_to_streamed_response_wrapper(
            actions.end,
        )
        self.kick = async_to_streamed_response_wrapper(
            actions.kick,
        )
        self.mute = async_to_streamed_response_wrapper(
            actions.mute,
        )
        self.unmute = async_to_streamed_response_wrapper(
            actions.unmute,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .texml_get_call_recording_response_body import TexmlGetCallRecordingResponseBody

__all__ = ["AccountRetrieveRecordingsJsonResponse"]


class AccountRetrieveRecordingsJsonResponse(BaseModel):
    end: Optional[int] = None
    """The number of the last element on the page, zero-indexed."""

    first_page_uri: Optional[str] = None
    """Relative uri to the first page of the query results"""

    next_page_uri: Optional[str] = None
    """Relative uri to the next page of the query results"""

    page: Optional[int] = None
    """Current page number, zero-indexed."""

    page_size: Optional[int] = None
    """The number of items on the page"""

    previous_page_uri: Optional[str] = None
    """Relative uri to the previous page of the query results"""

    recordings: Optional[List[TexmlGetCallRecordingResponseBody]] = None

    start: Optional[int] = None
    """The number of the first element on the page, zero-indexed."""

    uri: Optional[str] = None
    """The URI of the current page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel

__all__ = ["BucketListResponse", "Data"]


class Data(BaseModel):
    buckets: List[str]


class BucketListResponse(BaseModel):
    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .fine_tuning_job import FineTuningJob as FineTuningJob
from .job_create_params import JobCreateParams as JobCreateParams
from .job_list_response import JobListResponse as JobListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ActionRequirementInitiateResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the action requirement"""

    action_type: Optional[str] = None
    """The type of action required"""

    action_url: Optional[str] = None
    """Optional URL for the action"""

    cancel_reason: Optional[str] = None
    """Reason for cancellation if status is 'cancelled'"""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date-time indicating when the resource was created"""

    porting_order_id: Optional[str] = None
    """The ID of the porting order this action requirement belongs to"""

    record_type: Optional[Literal["porting_action_requirement"]] = None
    """Identifies the type of the resource"""

    requirement_type_id: Optional[str] = None
    """The ID of the requirement type"""

    status: Optional[Literal["created", "pending", "completed", "cancelled", "failed"]] = None
    """Current status of the action requirement"""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date-time indicating when the resource was updated"""


class ActionRequirementInitiateResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ConferenceSpeakStartedWebhookEvent", "Data", "DataPayload"]


class DataPayload(BaseModel):
    conference_id: Optional[str] = None
    """ID of the conference the text was spoken in."""

    connection_id: Optional[str] = None
    """Call Control App ID (formerly Telnyx connection ID) used in the call."""

    creator_call_session_id: Optional[str] = None
    """ID that is unique to the call session that started the conference."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 datetime of when the event occurred."""


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    event_type: Optional[Literal["conference.speak.started"]] = None
    """The type of event being delivered."""

    payload: Optional[DataPayload] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""


class ConferenceSpeakStartedWebhookEvent(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["ConferenceRetrieveConferencesResponse", "Conference"]


class Conference(BaseModel):
    account_sid: Optional[str] = None
    """The id of the account the resource belongs to."""

    api_version: Optional[str] = None
    """The version of the API that was used to make the request."""

    call_sid_ending_conference: Optional[str] = None
    """Caller ID, if present."""

    date_created: Optional[str] = None
    """The timestamp of when the resource was created."""

    date_updated: Optional[str] = None
    """The timestamp of when the resource was last updated."""

    friendly_name: Optional[str] = None
    """A string that you assigned to describe this conference room."""

    reason_conference_ended: Optional[
        Literal[
            "participant-with-end-conference-on-exit-left",
            "last-participant-left",
            "conference-ended-via-api",
            "time-exceeded",
        ]
    ] = None
    """The reason why a conference ended.

    When a conference is in progress, will be null.
    """

    region: Optional[str] = None
    """A string representing the region where the conference is hosted."""

    sid: Optional[str] = None
    """The unique identifier of the conference."""

    status: Optional[Literal["init", "in-progress", "completed"]] = None
    """The status of this conference."""

    subresource_uris: Optional[Dict[str, object]] = None
    """A list of related resources identified by their relative URIs."""

    uri: Optional[str] = None
    """The relative URI for this conference."""


class ConferenceRetrieveConferencesResponse(BaseModel):
    conferences: Optional[List[Conference]] = None

    end: Optional[int] = None
    """The number of the last element on the page, zero-indexed."""

    first_page_uri: Optional[str] = None
    """
    /v2/texml/Accounts/61bf923e-5e4d-4595-a110-56190ea18a1b/Conferences.json?Page=0&PageSize=1
    """

    next_page_uri: Optional[str] = None
    """
    /v2/texml/Accounts/61bf923e-5e4d-4595-a110-56190ea18a1b/Conferences.json?Page=1&PageSize=1&PageToken=MTY4AjgyNDkwNzIxMQ
    """

    page: Optional[int] = None
    """Current page number, zero-indexed."""

    page_size: Optional[int] = None
    """The number of items on the page"""

    start: Optional[int] = None
    """The number of the first element on the page, zero-indexed."""

    uri: Optional[str] = None
    """The URI of the current page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["HostedNumber"]


class HostedNumber(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    phone_number: Optional[str] = None
    """The messaging hosted phone number (+E.164 format)"""

    record_type: Optional[str] = None

    status: Optional[
        Literal[
            "deleted",
            "failed",
            "failed_activation",
            "failed_carrier_rejected",
            "failed_ineligible_carrier",
            "failed_number_already_hosted",
            "failed_number_not_found",
            "failed_ownership_verification",
            "failed_timeout",
            "pending",
            "provisioning",
            "successful",
        ]
    ] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    PortoutDetails,
    PortoutRetrieveResponse,
    PortoutUpdateStatusResponse,
    PortoutListRejectionCodesResponse,
)
from telnyx._utils import parse_datetime
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPortouts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        portout = client.portouts.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PortoutRetrieveResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.portouts.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        portout = response.parse()
        assert_matches_type(PortoutRetrieveResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.portouts.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            portout = response.parse()
            assert_matches_type(PortoutRetrieveResponse, portout, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.portouts.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        portout = client.portouts.list()
        assert_matches_type(SyncDefaultFlatPagination[PortoutDetails], portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        portout = client.portouts.list(
            filter={
                "carrier_name": "carrier_name",
                "country_code": "US",
                "country_code_in": ["CA", "US"],
                "foc_date": parse_datetime("2024-09-04T00:00:00.000Z"),
                "inserted_at": {
                    "gte": parse_datetime("2024-09-04T00:00:00.000Z"),
                    "lte": parse_datetime("2024-09-04T00:00:00.000Z"),
                },
                "phone_number": "+13035551212",
                "pon": "pon",
                "ported_out_at": {
                    "gte": parse_datetime("2024-09-04T00:00:00.000Z"),
                    "lte": parse_datetime("2024-09-04T00:00:00.000Z"),
                },
                "spid": "spid",
                "status": "pending",
                "status_in": ["pending"],
                "support_key": "PO_abc123",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[PortoutDetails], portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.portouts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        portout = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[PortoutDetails], portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.portouts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            portout = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[PortoutDetails], portout, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_rejection_codes(self, client: Telnyx) -> None:
        portout = client.portouts.list_rejection_codes(
            portout_id="329d6658-8f93-405d-862f-648776e8afd7",
        )
        assert_matches_type(PortoutListRejectionCodesResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_rejection_codes_with_all_params(self, client: Telnyx) -> None:
        portout = client.portouts.list_rejection_codes(
            portout_id="329d6658-8f93-405d-862f-648776e8afd7",
            filter={"code": 1002},
        )
        assert_matches_type(PortoutListRejectionCodesResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_rejection_codes(self, client: Telnyx) -> None:
        response = client.portouts.with_raw_response.list_rejection_codes(
            portout_id="329d6658-8f93-405d-862f-648776e8afd7",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        portout = response.parse()
        assert_matches_type(PortoutListRejectionCodesResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_rejection_codes(self, client: Telnyx) -> None:
        with client.portouts.with_streaming_response.list_rejection_codes(
            portout_id="329d6658-8f93-405d-862f-648776e8afd7",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            portout = response.parse()
            assert_matches_type(PortoutListRejectionCodesResponse, portout, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list_rejection_codes(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `portout_id` but received ''"):
            client.portouts.with_raw_response.list_rejection_codes(
                portout_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_status(self, client: Telnyx) -> None:
        portout = client.portouts.update_status(
            status="authorized",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            reason="I do not recognize this transaction",
        )
        assert_matches_type(PortoutUpdateStatusResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_status_with_all_params(self, client: Telnyx) -> None:
        portout = client.portouts.update_status(
            status="authorized",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            reason="I do not recognize this transaction",
            host_messaging=False,
        )
        assert_matches_type(PortoutUpdateStatusResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update_status(self, client: Telnyx) -> None:
        response = client.portouts.with_raw_response.update_status(
            status="authorized",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            reason="I do not recognize this transaction",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        portout = response.parse()
        assert_matches_type(PortoutUpdateStatusResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update_status(self, client: Telnyx) -> None:
        with client.portouts.with_streaming_response.update_status(
            status="authorized",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            reason="I do not recognize this transaction",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            portout = response.parse()
            assert_matches_type(PortoutUpdateStatusResponse, portout, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update_status(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.portouts.with_raw_response.update_status(
                status="authorized",
                id="",
                reason="I do not recognize this transaction",
            )


class TestAsyncPortouts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        portout = await async_client.portouts.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PortoutRetrieveResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.portouts.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        portout = await response.parse()
        assert_matches_type(PortoutRetrieveResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.portouts.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            portout = await response.parse()
            assert_matches_type(PortoutRetrieveResponse, portout, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.portouts.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        portout = await async_client.portouts.list()
        assert_matches_type(AsyncDefaultFlatPagination[PortoutDetails], portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        portout = await async_client.portouts.list(
            filter={
                "carrier_name": "carrier_name",
                "country_code": "US",
                "country_code_in": ["CA", "US"],
                "foc_date": parse_datetime("2024-09-04T00:00:00.000Z"),
                "inserted_at": {
                    "gte": parse_datetime("2024-09-04T00:00:00.000Z"),
                    "lte": parse_datetime("2024-09-04T00:00:00.000Z"),
                },
                "phone_number": "+13035551212",
                "pon": "pon",
                "ported_out_at": {
                    "gte": parse_datetime("2024-09-04T00:00:00.000Z"),
                    "lte": parse_datetime("2024-09-04T00:00:00.000Z"),
                },
                "spid": "spid",
                "status": "pending",
                "status_in": ["pending"],
                "support_key": "PO_abc123",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[PortoutDetails], portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.portouts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        portout = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[PortoutDetails], portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.portouts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            portout = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[PortoutDetails], portout, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_rejection_codes(self, async_client: AsyncTelnyx) -> None:
        portout = await async_client.portouts.list_rejection_codes(
            portout_id="329d6658-8f93-405d-862f-648776e8afd7",
        )
        assert_matches_type(PortoutListRejectionCodesResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_rejection_codes_with_all_params(self, async_client: AsyncTelnyx) -> None:
        portout = await async_client.portouts.list_rejection_codes(
            portout_id="329d6658-8f93-405d-862f-648776e8afd7",
            filter={"code": 1002},
        )
        assert_matches_type(PortoutListRejectionCodesResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_rejection_codes(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.portouts.with_raw_response.list_rejection_codes(
            portout_id="329d6658-8f93-405d-862f-648776e8afd7",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        portout = await response.parse()
        assert_matches_type(PortoutListRejectionCodesResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_rejection_codes(self, async_client: AsyncTelnyx) -> None:
        async with async_client.portouts.with_streaming_response.list_rejection_codes(
            portout_id="329d6658-8f93-405d-862f-648776e8afd7",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            portout = await response.parse()
            assert_matches_type(PortoutListRejectionCodesResponse, portout, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list_rejection_codes(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `portout_id` but received ''"):
            await async_client.portouts.with_raw_response.list_rejection_codes(
                portout_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_status(self, async_client: AsyncTelnyx) -> None:
        portout = await async_client.portouts.update_status(
            status="authorized",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            reason="I do not recognize this transaction",
        )
        assert_matches_type(PortoutUpdateStatusResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_status_with_all_params(self, async_client: AsyncTelnyx) -> None:
        portout = await async_client.portouts.update_status(
            status="authorized",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            reason="I do not recognize this transaction",
            host_messaging=False,
        )
        assert_matches_type(PortoutUpdateStatusResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update_status(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.portouts.with_raw_response.update_status(
            status="authorized",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            reason="I do not recognize this transaction",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        portout = await response.parse()
        assert_matches_type(PortoutUpdateStatusResponse, portout, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update_status(self, async_client: AsyncTelnyx) -> None:
        async with async_client.portouts.with_streaming_response.update_status(
            status="authorized",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            reason="I do not recognize this transaction",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            portout = await response.parse()
            assert_matches_type(PortoutUpdateStatusResponse, portout, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update_status(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.portouts.with_raw_response.update_status(
                status="authorized",
                id="",
                reason="I do not recognize this transaction",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .outbound_voice_profile import OutboundVoiceProfile

__all__ = ["OutboundVoiceProfileRetrieveResponse"]


class OutboundVoiceProfileRetrieveResponse(BaseModel):
    data: Optional[OutboundVoiceProfile] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import TypedDict

__all__ = ["RequirementGroupUpdateParams", "RegulatoryRequirement"]


class RequirementGroupUpdateParams(TypedDict, total=False):
    customer_reference: str
    """Reference for the customer"""

    regulatory_requirements: Iterable[RegulatoryRequirement]


class RegulatoryRequirement(TypedDict, total=False):
    field_value: str
    """New value for the regulatory requirement"""

    requirement_id: str
    """Unique identifier for the regulatory requirement"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from ..._types import SequenceNotStr

__all__ = ["VerificationCodeSendParams"]


class VerificationCodeSendParams(TypedDict, total=False):
    phone_numbers: SequenceNotStr[str]

    verification_method: Literal["sms", "call"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .outbound_message_payload import OutboundMessagePayload
from .shared.inbound_message_payload import InboundMessagePayload

__all__ = ["MessageRetrieveResponse", "Data"]

Data: TypeAlias = Annotated[
    Union[OutboundMessagePayload, InboundMessagePayload], PropertyInfo(discriminator="direction")
]


class MessageRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["SipHeader"]


class SipHeader(BaseModel):
    name: Literal["User-to-User"]
    """The name of the header to add."""

    value: str
    """The value of the header."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .mobile_phone_number import MobilePhoneNumber

__all__ = ["MobilePhoneNumberRetrieveResponse"]


class MobilePhoneNumberRetrieveResponse(BaseModel):
    data: Optional[MobilePhoneNumber] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types.texml.accounts.calls import RecordingRecordingSidJsonResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRecordings:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_recording_sid_json(self, client: Telnyx) -> None:
        recording = client.texml.accounts.calls.recordings.recording_sid_json(
            recording_sid="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            account_sid="account_sid",
            call_sid="call_sid",
        )
        assert_matches_type(RecordingRecordingSidJsonResponse, recording, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_recording_sid_json_with_all_params(self, client: Telnyx) -> None:
        recording = client.texml.accounts.calls.recordings.recording_sid_json(
            recording_sid="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            account_sid="account_sid",
            call_sid="call_sid",
            status="paused",
        )
        assert_matches_type(RecordingRecordingSidJsonResponse, recording, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_recording_sid_json(self, client: Telnyx) -> None:
        response = client.texml.accounts.calls.recordings.with_raw_response.recording_sid_json(
            recording_sid="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            account_sid="account_sid",
            call_sid="call_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        recording = response.parse()
        assert_matches_type(RecordingRecordingSidJsonResponse, recording, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_recording_sid_json(self, client: Telnyx) -> None:
        with client.texml.accounts.calls.recordings.with_streaming_response.recording_sid_json(
            recording_sid="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            account_sid="account_sid",
            call_sid="call_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            recording = response.parse()
            assert_matches_type(RecordingRecordingSidJsonResponse, recording, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_recording_sid_json(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            client.texml.accounts.calls.recordings.with_raw_response.recording_sid_json(
                recording_sid="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
                account_sid="",
                call_sid="call_sid",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `call_sid` but received ''"):
            client.texml.accounts.calls.recordings.with_raw_response.recording_sid_json(
                recording_sid="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
                account_sid="account_sid",
                call_sid="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `recording_sid` but received ''"):
            client.texml.accounts.calls.recordings.with_raw_response.recording_sid_json(
                recording_sid="",
                account_sid="account_sid",
                call_sid="call_sid",
            )


class TestAsyncRecordings:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_recording_sid_json(self, async_client: AsyncTelnyx) -> None:
        recording = await async_client.texml.accounts.calls.recordings.recording_sid_json(
            recording_sid="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            account_sid="account_sid",
            call_sid="call_sid",
        )
        assert_matches_type(RecordingRecordingSidJsonResponse, recording, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_recording_sid_json_with_all_params(self, async_client: AsyncTelnyx) -> None:
        recording = await async_client.texml.accounts.calls.recordings.recording_sid_json(
            recording_sid="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            account_sid="account_sid",
            call_sid="call_sid",
            status="paused",
        )
        assert_matches_type(RecordingRecordingSidJsonResponse, recording, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_recording_sid_json(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.texml.accounts.calls.recordings.with_raw_response.recording_sid_json(
            recording_sid="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            account_sid="account_sid",
            call_sid="call_sid",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        recording = await response.parse()
        assert_matches_type(RecordingRecordingSidJsonResponse, recording, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_recording_sid_json(self, async_client: AsyncTelnyx) -> None:
        async with async_client.texml.accounts.calls.recordings.with_streaming_response.recording_sid_json(
            recording_sid="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            account_sid="account_sid",
            call_sid="call_sid",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            recording = await response.parse()
            assert_matches_type(RecordingRecordingSidJsonResponse, recording, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_recording_sid_json(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `account_sid` but received ''"):
            await async_client.texml.accounts.calls.recordings.with_raw_response.recording_sid_json(
                recording_sid="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
                account_sid="",
                call_sid="call_sid",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `call_sid` but received ''"):
            await async_client.texml.accounts.calls.recordings.with_raw_response.recording_sid_json(
                recording_sid="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
                account_sid="account_sid",
                call_sid="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `recording_sid` but received ''"):
            await async_client.texml.accounts.calls.recordings.with_raw_response.recording_sid_json(
                recording_sid="",
                account_sid="account_sid",
                call_sid="call_sid",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = [
    "ConferenceRecordingSavedWebhookEvent",
    "Data",
    "DataPayload",
    "DataPayloadPublicRecordingURLs",
    "DataPayloadRecordingURLs",
]


class DataPayloadPublicRecordingURLs(BaseModel):
    """Recording URLs in requested format.

    The URL is valid for as long as the file exists. For security purposes, this feature is activated on a per request basis.  Please contact customer support with your Account ID to request activation.
    """

    mp3: Optional[str] = None
    """Recording URL in requested `mp3` format."""

    wav: Optional[str] = None
    """Recording URL in requested `wav` format."""


class DataPayloadRecordingURLs(BaseModel):
    """Recording URLs in requested format.

    These URLs are valid for 10 minutes. After 10 minutes, you may retrieve recordings via API using Reports -> Call Recordings documentation, or via Mission Control under Reporting -> Recordings.
    """

    mp3: Optional[str] = None
    """Recording URL in requested `mp3` format."""

    wav: Optional[str] = None
    """Recording URL in requested `wav` format."""


class DataPayload(BaseModel):
    call_control_id: Optional[str] = None
    """Participant's call ID used to issue commands via Call Control API."""

    call_session_id: Optional[str] = None
    """
    ID that is unique to the call session and can be used to correlate webhook
    events. Call session is a group of related call legs that logically belong to
    the same phone call, e.g. an inbound and outbound leg of a transferred call.
    """

    channels: Optional[Literal["single", "dual"]] = None
    """Whether recording was recorded in `single` or `dual` channel."""

    client_state: Optional[str] = None
    """State received from a command."""

    conference_id: Optional[str] = None
    """ID of the conference that is being recorded."""

    connection_id: Optional[str] = None
    """Call Control App ID (formerly Telnyx connection ID) used in the call."""

    format: Optional[Literal["wav", "mp3"]] = None
    """The audio file format used when storing the call recording.

    Can be either `mp3` or `wav`.
    """

    public_recording_urls: Optional[DataPayloadPublicRecordingURLs] = None
    """Recording URLs in requested format.

    The URL is valid for as long as the file exists. For security purposes, this
    feature is activated on a per request basis. Please contact customer support
    with your Account ID to request activation.
    """

    recording_ended_at: Optional[datetime] = None
    """ISO 8601 datetime of when recording ended."""

    recording_id: Optional[str] = None
    """ID of the conference recording."""

    recording_started_at: Optional[datetime] = None
    """ISO 8601 datetime of when recording started."""

    recording_urls: Optional[DataPayloadRecordingURLs] = None
    """Recording URLs in requested format.

    These URLs are valid for 10 minutes. After 10 minutes, you may retrieve
    recordings via API using Reports -> Call Recordings documentation, or via
    Mission Control under Reporting -> Recordings.
    """


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    event_type: Optional[Literal["conference.recording.saved"]] = None
    """The type of event being delivered."""

    payload: Optional[DataPayload] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""


class ConferenceRecordingSavedWebhookEvent(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .credential_connections import (
    CredentialConnectionsResource,
    AsyncCredentialConnectionsResource,
    CredentialConnectionsResourceWithRawResponse,
    AsyncCredentialConnectionsResourceWithRawResponse,
    CredentialConnectionsResourceWithStreamingResponse,
    AsyncCredentialConnectionsResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "CredentialConnectionsResource",
    "AsyncCredentialConnectionsResource",
    "CredentialConnectionsResourceWithRawResponse",
    "AsyncCredentialConnectionsResourceWithRawResponse",
    "CredentialConnectionsResourceWithStreamingResponse",
    "AsyncCredentialConnectionsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["PortingListUkCarriersResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the UK carrier."""

    alternative_cupids: Optional[List[str]] = None
    """Alternative CUPIDs of the carrier."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    cupid: Optional[str] = None
    """The CUPID of the carrier.

    This is a 3 digit number code that identifies the carrier in the UK.
    """

    name: Optional[str] = None
    """The name of the carrier."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""


class PortingListUkCarriersResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, FileTypes, omit, not_given
from ..._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from ..._compat import cached_property
from ...types.ai import audio_transcribe_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.ai.audio_transcribe_response import AudioTranscribeResponse

__all__ = ["AudioResource", "AsyncAudioResource"]


class AudioResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AudioResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AudioResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AudioResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AudioResourceWithStreamingResponse(self)

    def transcribe(
        self,
        *,
        model: Literal["distil-whisper/distil-large-v2", "openai/whisper-large-v3-turbo"],
        file: FileTypes | Omit = omit,
        file_url: str | Omit = omit,
        response_format: Literal["json", "verbose_json"] | Omit = omit,
        timestamp_granularities: Literal["segment"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudioTranscribeResponse:
        """Transcribe speech to text.

        This endpoint is consistent with the
        [OpenAI Transcription API](https://platform.openai.com/docs/api-reference/audio/createTranscription)
        and may be used with the OpenAI JS or Python SDK.

        Args:
          model: ID of the model to use. `distil-whisper/distil-large-v2` is lower latency but
              English-only. `openai/whisper-large-v3-turbo` is multi-lingual but slightly
              higher latency.

          file: The audio file object to transcribe, in one of these formats: flac, mp3, mp4,
              mpeg, mpga, m4a, ogg, wav, or webm. File uploads are limited to 100 MB. Cannot
              be used together with `file_url`

          file_url: Link to audio file in one of these formats: flac, mp3, mp4, mpeg, mpga, m4a,
              ogg, wav, or webm. Support for hosted files is limited to 100MB. Cannot be used
              together with `file`

          response_format: The format of the transcript output. Use `verbose_json` to take advantage of
              timestamps.

          timestamp_granularities: The timestamp granularities to populate for this transcription.
              `response_format` must be set verbose_json to use timestamp granularities.
              Currently `segment` is supported.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "model": model,
                "file": file,
                "file_url": file_url,
                "response_format": response_format,
                "timestamp_granularities": timestamp_granularities,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/ai/audio/transcriptions",
            body=maybe_transform(body, audio_transcribe_params.AudioTranscribeParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AudioTranscribeResponse,
        )


class AsyncAudioResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAudioResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAudioResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAudioResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAudioResourceWithStreamingResponse(self)

    async def transcribe(
        self,
        *,
        model: Literal["distil-whisper/distil-large-v2", "openai/whisper-large-v3-turbo"],
        file: FileTypes | Omit = omit,
        file_url: str | Omit = omit,
        response_format: Literal["json", "verbose_json"] | Omit = omit,
        timestamp_granularities: Literal["segment"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AudioTranscribeResponse:
        """Transcribe speech to text.

        This endpoint is consistent with the
        [OpenAI Transcription API](https://platform.openai.com/docs/api-reference/audio/createTranscription)
        and may be used with the OpenAI JS or Python SDK.

        Args:
          model: ID of the model to use. `distil-whisper/distil-large-v2` is lower latency but
              English-only. `openai/whisper-large-v3-turbo` is multi-lingual but slightly
              higher latency.

          file: The audio file object to transcribe, in one of these formats: flac, mp3, mp4,
              mpeg, mpga, m4a, ogg, wav, or webm. File uploads are limited to 100 MB. Cannot
              be used together with `file_url`

          file_url: Link to audio file in one of these formats: flac, mp3, mp4, mpeg, mpga, m4a,
              ogg, wav, or webm. Support for hosted files is limited to 100MB. Cannot be used
              together with `file`

          response_format: The format of the transcript output. Use `verbose_json` to take advantage of
              timestamps.

          timestamp_granularities: The timestamp granularities to populate for this transcription.
              `response_format` must be set verbose_json to use timestamp granularities.
              Currently `segment` is supported.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "model": model,
                "file": file,
                "file_url": file_url,
                "response_format": response_format,
                "timestamp_granularities": timestamp_granularities,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/ai/audio/transcriptions",
            body=await async_maybe_transform(body, audio_transcribe_params.AudioTranscribeParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AudioTranscribeResponse,
        )


class AudioResourceWithRawResponse:
    def __init__(self, audio: AudioResource) -> None:
        self._audio = audio

        self.transcribe = to_raw_response_wrapper(
            audio.transcribe,
        )


class AsyncAudioResourceWithRawResponse:
    def __init__(self, audio: AsyncAudioResource) -> None:
        self._audio = audio

        self.transcribe = async_to_raw_response_wrapper(
            audio.transcribe,
        )


class AudioResourceWithStreamingResponse:
    def __init__(self, audio: AudioResource) -> None:
        self._audio = audio

        self.transcribe = to_streamed_response_wrapper(
            audio.transcribe,
        )


class AsyncAudioResourceWithStreamingResponse:
    def __init__(self, audio: AsyncAudioResource) -> None:
        self._audio = audio

        self.transcribe = async_to_streamed_response_wrapper(
            audio.transcribe,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .reporting import (
    ReportingResource,
    AsyncReportingResource,
    ReportingResourceWithRawResponse,
    AsyncReportingResourceWithRawResponse,
    ReportingResourceWithStreamingResponse,
    AsyncReportingResourceWithStreamingResponse,
)
from .usage_reports import (
    UsageReportsResource,
    AsyncUsageReportsResource,
    UsageReportsResourceWithRawResponse,
    AsyncUsageReportsResourceWithRawResponse,
    UsageReportsResourceWithStreamingResponse,
    AsyncUsageReportsResourceWithStreamingResponse,
)
from .batch_detail_records import (
    BatchDetailRecordsResource,
    AsyncBatchDetailRecordsResource,
    BatchDetailRecordsResourceWithRawResponse,
    AsyncBatchDetailRecordsResourceWithRawResponse,
    BatchDetailRecordsResourceWithStreamingResponse,
    AsyncBatchDetailRecordsResourceWithStreamingResponse,
)

__all__ = [
    "BatchDetailRecordsResource",
    "AsyncBatchDetailRecordsResource",
    "BatchDetailRecordsResourceWithRawResponse",
    "AsyncBatchDetailRecordsResourceWithRawResponse",
    "BatchDetailRecordsResourceWithStreamingResponse",
    "AsyncBatchDetailRecordsResourceWithStreamingResponse",
    "UsageReportsResource",
    "AsyncUsageReportsResource",
    "UsageReportsResourceWithRawResponse",
    "AsyncUsageReportsResourceWithRawResponse",
    "UsageReportsResourceWithStreamingResponse",
    "AsyncUsageReportsResourceWithStreamingResponse",
    "ReportingResource",
    "AsyncReportingResource",
    "ReportingResourceWithRawResponse",
    "AsyncReportingResourceWithRawResponse",
    "ReportingResourceWithStreamingResponse",
    "AsyncReportingResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .usage import (
    UsageResource,
    AsyncUsageResource,
    UsageResourceWithRawResponse,
    AsyncUsageResourceWithRawResponse,
    UsageResourceWithStreamingResponse,
    AsyncUsageResourceWithStreamingResponse,
)
from .buckets import (
    BucketsResource,
    AsyncBucketsResource,
    BucketsResourceWithRawResponse,
    AsyncBucketsResourceWithRawResponse,
    BucketsResourceWithStreamingResponse,
    AsyncBucketsResourceWithStreamingResponse,
)
from .ssl_certificate import (
    SslCertificateResource,
    AsyncSslCertificateResource,
    SslCertificateResourceWithRawResponse,
    AsyncSslCertificateResourceWithRawResponse,
    SslCertificateResourceWithStreamingResponse,
    AsyncSslCertificateResourceWithStreamingResponse,
)

__all__ = [
    "SslCertificateResource",
    "AsyncSslCertificateResource",
    "SslCertificateResourceWithRawResponse",
    "AsyncSslCertificateResourceWithRawResponse",
    "SslCertificateResourceWithStreamingResponse",
    "AsyncSslCertificateResourceWithStreamingResponse",
    "UsageResource",
    "AsyncUsageResource",
    "UsageResourceWithRawResponse",
    "AsyncUsageResourceWithRawResponse",
    "UsageResourceWithStreamingResponse",
    "AsyncUsageResourceWithStreamingResponse",
    "BucketsResource",
    "AsyncBucketsResource",
    "BucketsResourceWithRawResponse",
    "AsyncBucketsResourceWithRawResponse",
    "BucketsResourceWithStreamingResponse",
    "AsyncBucketsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from .._models import BaseModel
from .verify_profile_message_template_response import VerifyProfileMessageTemplateResponse

__all__ = ["VerifyProfileRetrieveTemplatesResponse"]


class VerifyProfileRetrieveTemplatesResponse(BaseModel):
    """A list of Verify profile message templates"""

    data: List[VerifyProfileMessageTemplateResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from ...._models import BaseModel

__all__ = ["QueueCreateResponse"]


class QueueCreateResponse(BaseModel):
    account_sid: Optional[str] = None
    """The id of the account the resource belongs to."""

    average_wait_time: Optional[int] = None
    """The average wait time in seconds for members in the queue."""

    current_size: Optional[int] = None
    """The current number of members in the queue."""

    date_created: Optional[str] = None
    """The timestamp of when the resource was created."""

    date_updated: Optional[str] = None
    """The timestamp of when the resource was last updated."""

    max_size: Optional[int] = None
    """The maximum size of the queue."""

    sid: Optional[str] = None
    """The unique identifier of the queue."""

    subresource_uris: Optional[Dict[str, object]] = None
    """A list of related resources identified by their relative URIs."""

    uri: Optional[str] = None
    """The relative URI for this queue."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .meta import Meta as Meta
from .run_list_params import RunListParams as RunListParams
from .run_trigger_params import RunTriggerParams as RunTriggerParams
from .run_trigger_response import RunTriggerResponse as RunTriggerResponse
from .paginated_test_run_list import PaginatedTestRunList as PaginatedTestRunList



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["TestStatus"]

TestStatus: TypeAlias = Literal["pending", "starting", "running", "passed", "failed", "error"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = [
    "SubNumberOrderUpdateRequirementGroupResponse",
    "Data",
    "DataPhoneNumber",
    "DataPhoneNumberRegulatoryRequirement",
    "DataRegulatoryRequirement",
]


class DataPhoneNumberRegulatoryRequirement(BaseModel):
    field_type: Optional[str] = None

    field_value: Optional[str] = None

    requirement_id: Optional[str] = None

    status: Optional[str] = None


class DataPhoneNumber(BaseModel):
    id: Optional[str] = None

    bundle_id: Optional[str] = None

    country_code: Optional[str] = None

    phone_number: Optional[str] = None

    phone_number_type: Optional[str] = None

    record_type: Optional[str] = None

    regulatory_requirements: Optional[List[DataPhoneNumberRegulatoryRequirement]] = None

    requirements_met: Optional[bool] = None

    requirements_status: Optional[str] = None

    status: Optional[str] = None


class DataRegulatoryRequirement(BaseModel):
    field_type: Optional[str] = None

    record_type: Optional[str] = None

    requirement_id: Optional[str] = None


class Data(BaseModel):
    id: Optional[str] = None

    country_code: Optional[str] = None

    created_at: Optional[datetime] = None

    customer_reference: Optional[str] = None

    is_block_sub_number_order: Optional[bool] = None

    order_request_id: Optional[str] = None

    phone_number_type: Optional[str] = None

    phone_numbers: Optional[List[DataPhoneNumber]] = None

    phone_numbers_count: Optional[int] = None

    record_type: Optional[str] = None

    regulatory_requirements: Optional[List[DataRegulatoryRequirement]] = None

    requirements_met: Optional[bool] = None

    status: Optional[str] = None

    updated_at: Optional[datetime] = None


class SubNumberOrderUpdateRequirementGroupResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.sim_cards import (
    action_list_params,
    action_set_public_ip_params,
    action_bulk_set_public_ips_params,
    action_validate_registration_codes_params,
)
from ...types.sim_cards.sim_card_action import SimCardAction
from ...types.sim_cards.action_enable_response import ActionEnableResponse
from ...types.sim_cards.action_disable_response import ActionDisableResponse
from ...types.sim_cards.action_retrieve_response import ActionRetrieveResponse
from ...types.sim_cards.action_set_standby_response import ActionSetStandbyResponse
from ...types.sim_cards.action_set_public_ip_response import ActionSetPublicIPResponse
from ...types.sim_cards.action_remove_public_ip_response import ActionRemovePublicIPResponse
from ...types.sim_cards.action_bulk_set_public_ips_response import ActionBulkSetPublicIPsResponse
from ...types.sim_cards.action_validate_registration_codes_response import ActionValidateRegistrationCodesResponse

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRetrieveResponse:
        """
        This API fetches detailed information about a SIM card action to follow-up on an
        existing asynchronous operation.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/sim_card_actions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: action_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[SimCardAction]:
        """This API lists a paginated collection of SIM card actions.

        It enables exploring
        a collection of existing asynchronous operations using specific filters.

        Args:
          filter: Consolidated filter parameter for SIM card actions (deepObject style).
              Originally: filter[sim_card_id], filter[status],
              filter[bulk_sim_card_action_id], filter[action_type]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/sim_card_actions",
            page=SyncDefaultFlatPagination[SimCardAction],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    action_list_params.ActionListParams,
                ),
            ),
            model=SimCardAction,
        )

    def bulk_set_public_ips(
        self,
        *,
        sim_card_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionBulkSetPublicIPsResponse:
        """
        This API triggers an asynchronous operation to set a public IP for each of the
        specified SIM cards.<br/> For each SIM Card a SIM Card Action will be generated.
        The status of the SIM Card Action can be followed through the
        [List SIM Card Action](https://developers.telnyx.com/api/wireless/list-sim-card-actions)
        API.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/sim_cards/actions/bulk_set_public_ips",
            body=maybe_transform(
                {"sim_card_ids": sim_card_ids}, action_bulk_set_public_ips_params.ActionBulkSetPublicIPsParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionBulkSetPublicIPsResponse,
        )

    def disable(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionDisableResponse:
        """
        This API disables a SIM card, disconnecting it from the network and making it
        impossible to consume data.<br/> The API will trigger an asynchronous operation
        called a SIM Card Action. Transitioning to the disabled state may take a period
        of time. The status of the SIM Card Action can be followed through the
        [List SIM Card Action](https://developers.telnyx.com/api/wireless/list-sim-card-actions)
        API.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/sim_cards/{id}/actions/disable",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionDisableResponse,
        )

    def enable(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionEnableResponse:
        """
        This API enables a SIM card, connecting it to the network and making it possible
        to consume data.<br/> To enable a SIM card, it must be associated with a SIM
        card group.<br/> The API will trigger an asynchronous operation called a SIM
        Card Action. Transitioning to the enabled state may take a period of time. The
        status of the SIM Card Action can be followed through the
        [List SIM Card Action](https://developers.telnyx.com/api/wireless/list-sim-card-actions)
        API.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/sim_cards/{id}/actions/enable",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionEnableResponse,
        )

    def remove_public_ip(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRemovePublicIPResponse:
        """This API removes an existing public IP from a SIM card.

        <br/><br/> The API will
        trigger an asynchronous operation called a SIM Card Action. The status of the
        SIM Card Action can be followed through the
        [List SIM Card Action](https://developers.telnyx.com/api/wireless/list-sim-card-actions)
        API.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/sim_cards/{id}/actions/remove_public_ip",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRemovePublicIPResponse,
        )

    def set_public_ip(
        self,
        id: str,
        *,
        region_code: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSetPublicIPResponse:
        """
        This API makes a SIM card reachable on the public internet by mapping a random
        public IP to the SIM card. <br/><br/> The API will trigger an asynchronous
        operation called a SIM Card Action. The status of the SIM Card Action can be
        followed through the
        [List SIM Card Action](https://developers.telnyx.com/api/wireless/list-sim-card-actions)
        API. <br/><br/> Setting a Public IP to a SIM Card incurs a charge and will only
        succeed if the account has sufficient funds.

        Args:
          region_code: The code of the region where the public IP should be assigned. A list of
              available regions can be found at the regions endpoint

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/sim_cards/{id}/actions/set_public_ip",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"region_code": region_code}, action_set_public_ip_params.ActionSetPublicIPParams
                ),
            ),
            cast_to=ActionSetPublicIPResponse,
        )

    def set_standby(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSetStandbyResponse:
        """
        The SIM card will be able to connect to the network once the process to set it
        to standby has been completed, thus making it possible to consume data.<br/> To
        set a SIM card to standby, it must be associated with SIM card group.<br/> The
        API will trigger an asynchronous operation called a SIM Card Action.
        Transitioning to the standby state may take a period of time. The status of the
        SIM Card Action can be followed through the
        [List SIM Card Action](https://developers.telnyx.com/api/wireless/list-sim-card-actions)
        API.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/sim_cards/{id}/actions/set_standby",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionSetStandbyResponse,
        )

    def validate_registration_codes(
        self,
        *,
        registration_codes: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionValidateRegistrationCodesResponse:
        """
        It validates whether SIM card registration codes are valid or not.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/sim_cards/actions/validate_registration_codes",
            body=maybe_transform(
                {"registration_codes": registration_codes},
                action_validate_registration_codes_params.ActionValidateRegistrationCodesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionValidateRegistrationCodesResponse,
        )


class AsyncActionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRetrieveResponse:
        """
        This API fetches detailed information about a SIM card action to follow-up on an
        existing asynchronous operation.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/sim_card_actions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: action_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SimCardAction, AsyncDefaultFlatPagination[SimCardAction]]:
        """This API lists a paginated collection of SIM card actions.

        It enables exploring
        a collection of existing asynchronous operations using specific filters.

        Args:
          filter: Consolidated filter parameter for SIM card actions (deepObject style).
              Originally: filter[sim_card_id], filter[status],
              filter[bulk_sim_card_action_id], filter[action_type]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/sim_card_actions",
            page=AsyncDefaultFlatPagination[SimCardAction],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    action_list_params.ActionListParams,
                ),
            ),
            model=SimCardAction,
        )

    async def bulk_set_public_ips(
        self,
        *,
        sim_card_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionBulkSetPublicIPsResponse:
        """
        This API triggers an asynchronous operation to set a public IP for each of the
        specified SIM cards.<br/> For each SIM Card a SIM Card Action will be generated.
        The status of the SIM Card Action can be followed through the
        [List SIM Card Action](https://developers.telnyx.com/api/wireless/list-sim-card-actions)
        API.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/sim_cards/actions/bulk_set_public_ips",
            body=await async_maybe_transform(
                {"sim_card_ids": sim_card_ids}, action_bulk_set_public_ips_params.ActionBulkSetPublicIPsParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionBulkSetPublicIPsResponse,
        )

    async def disable(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionDisableResponse:
        """
        This API disables a SIM card, disconnecting it from the network and making it
        impossible to consume data.<br/> The API will trigger an asynchronous operation
        called a SIM Card Action. Transitioning to the disabled state may take a period
        of time. The status of the SIM Card Action can be followed through the
        [List SIM Card Action](https://developers.telnyx.com/api/wireless/list-sim-card-actions)
        API.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/sim_cards/{id}/actions/disable",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionDisableResponse,
        )

    async def enable(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionEnableResponse:
        """
        This API enables a SIM card, connecting it to the network and making it possible
        to consume data.<br/> To enable a SIM card, it must be associated with a SIM
        card group.<br/> The API will trigger an asynchronous operation called a SIM
        Card Action. Transitioning to the enabled state may take a period of time. The
        status of the SIM Card Action can be followed through the
        [List SIM Card Action](https://developers.telnyx.com/api/wireless/list-sim-card-actions)
        API.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/sim_cards/{id}/actions/enable",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionEnableResponse,
        )

    async def remove_public_ip(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRemovePublicIPResponse:
        """This API removes an existing public IP from a SIM card.

        <br/><br/> The API will
        trigger an asynchronous operation called a SIM Card Action. The status of the
        SIM Card Action can be followed through the
        [List SIM Card Action](https://developers.telnyx.com/api/wireless/list-sim-card-actions)
        API.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/sim_cards/{id}/actions/remove_public_ip",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRemovePublicIPResponse,
        )

    async def set_public_ip(
        self,
        id: str,
        *,
        region_code: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSetPublicIPResponse:
        """
        This API makes a SIM card reachable on the public internet by mapping a random
        public IP to the SIM card. <br/><br/> The API will trigger an asynchronous
        operation called a SIM Card Action. The status of the SIM Card Action can be
        followed through the
        [List SIM Card Action](https://developers.telnyx.com/api/wireless/list-sim-card-actions)
        API. <br/><br/> Setting a Public IP to a SIM Card incurs a charge and will only
        succeed if the account has sufficient funds.

        Args:
          region_code: The code of the region where the public IP should be assigned. A list of
              available regions can be found at the regions endpoint

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/sim_cards/{id}/actions/set_public_ip",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"region_code": region_code}, action_set_public_ip_params.ActionSetPublicIPParams
                ),
            ),
            cast_to=ActionSetPublicIPResponse,
        )

    async def set_standby(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionSetStandbyResponse:
        """
        The SIM card will be able to connect to the network once the process to set it
        to standby has been completed, thus making it possible to consume data.<br/> To
        set a SIM card to standby, it must be associated with SIM card group.<br/> The
        API will trigger an asynchronous operation called a SIM Card Action.
        Transitioning to the standby state may take a period of time. The status of the
        SIM Card Action can be followed through the
        [List SIM Card Action](https://developers.telnyx.com/api/wireless/list-sim-card-actions)
        API.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/sim_cards/{id}/actions/set_standby",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionSetStandbyResponse,
        )

    async def validate_registration_codes(
        self,
        *,
        registration_codes: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionValidateRegistrationCodesResponse:
        """
        It validates whether SIM card registration codes are valid or not.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/sim_cards/actions/validate_registration_codes",
            body=await async_maybe_transform(
                {"registration_codes": registration_codes},
                action_validate_registration_codes_params.ActionValidateRegistrationCodesParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionValidateRegistrationCodesResponse,
        )


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.retrieve = to_raw_response_wrapper(
            actions.retrieve,
        )
        self.list = to_raw_response_wrapper(
            actions.list,
        )
        self.bulk_set_public_ips = to_raw_response_wrapper(
            actions.bulk_set_public_ips,
        )
        self.disable = to_raw_response_wrapper(
            actions.disable,
        )
        self.enable = to_raw_response_wrapper(
            actions.enable,
        )
        self.remove_public_ip = to_raw_response_wrapper(
            actions.remove_public_ip,
        )
        self.set_public_ip = to_raw_response_wrapper(
            actions.set_public_ip,
        )
        self.set_standby = to_raw_response_wrapper(
            actions.set_standby,
        )
        self.validate_registration_codes = to_raw_response_wrapper(
            actions.validate_registration_codes,
        )


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.retrieve = async_to_raw_response_wrapper(
            actions.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            actions.list,
        )
        self.bulk_set_public_ips = async_to_raw_response_wrapper(
            actions.bulk_set_public_ips,
        )
        self.disable = async_to_raw_response_wrapper(
            actions.disable,
        )
        self.enable = async_to_raw_response_wrapper(
            actions.enable,
        )
        self.remove_public_ip = async_to_raw_response_wrapper(
            actions.remove_public_ip,
        )
        self.set_public_ip = async_to_raw_response_wrapper(
            actions.set_public_ip,
        )
        self.set_standby = async_to_raw_response_wrapper(
            actions.set_standby,
        )
        self.validate_registration_codes = async_to_raw_response_wrapper(
            actions.validate_registration_codes,
        )


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.retrieve = to_streamed_response_wrapper(
            actions.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            actions.list,
        )
        self.bulk_set_public_ips = to_streamed_response_wrapper(
            actions.bulk_set_public_ips,
        )
        self.disable = to_streamed_response_wrapper(
            actions.disable,
        )
        self.enable = to_streamed_response_wrapper(
            actions.enable,
        )
        self.remove_public_ip = to_streamed_response_wrapper(
            actions.remove_public_ip,
        )
        self.set_public_ip = to_streamed_response_wrapper(
            actions.set_public_ip,
        )
        self.set_standby = to_streamed_response_wrapper(
            actions.set_standby,
        )
        self.validate_registration_codes = to_streamed_response_wrapper(
            actions.validate_registration_codes,
        )


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.retrieve = async_to_streamed_response_wrapper(
            actions.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            actions.list,
        )
        self.bulk_set_public_ips = async_to_streamed_response_wrapper(
            actions.bulk_set_public_ips,
        )
        self.disable = async_to_streamed_response_wrapper(
            actions.disable,
        )
        self.enable = async_to_streamed_response_wrapper(
            actions.enable,
        )
        self.remove_public_ip = async_to_streamed_response_wrapper(
            actions.remove_public_ip,
        )
        self.set_public_ip = async_to_streamed_response_wrapper(
            actions.set_public_ip,
        )
        self.set_standby = async_to_streamed_response_wrapper(
            actions.set_standby,
        )
        self.validate_registration_codes = async_to_streamed_response_wrapper(
            actions.validate_registration_codes,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["TfPhoneNumber"]


class TfPhoneNumber(BaseModel):
    """A phone number"""

    phone_number: str = FieldInfo(alias="phoneNumber")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ReportListParams", "Filter"]


class ReportListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[report_type], filter[status]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[report_type], filter[status]
    """

    report_type: Literal["export_porting_orders_csv"]
    """Filter reports of a specific type"""

    status: Literal["pending", "completed"]
    """Filter reports of a specific status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["InferenceEmbeddingWebhookToolParams", "BodyParameters", "Header", "PathParameters", "QueryParameters"]


class BodyParameters(BaseModel):
    """The body parameters the webhook tool accepts, described as a JSON Schema object.

    These parameters will be passed to the webhook as the body of the request. See the [JSON Schema reference](https://json-schema.org/understanding-json-schema) for documentation about the format
    """

    properties: Optional[Dict[str, object]] = None
    """The properties of the body parameters."""

    required: Optional[List[str]] = None
    """The required properties of the body parameters."""

    type: Optional[Literal["object"]] = None


class Header(BaseModel):
    name: Optional[str] = None

    value: Optional[str] = None
    """The value of the header.

    Note that we support mustache templating for the value. For example you can use
    `Bearer {{#integration_secret}}test-secret{{/integration_secret}}` to pass the
    value of the integration secret as the bearer token.
    [Telnyx signature headers](https://developers.telnyx.com/docs/voice/programmable-voice/voice-api-webhooks)
    will be automatically added to the request.
    """


class PathParameters(BaseModel):
    """The path parameters the webhook tool accepts, described as a JSON Schema object.

    These parameters will be passed to the webhook as the path of the request if the URL contains a placeholder for a value. See the [JSON Schema reference](https://json-schema.org/understanding-json-schema) for documentation about the format
    """

    properties: Optional[Dict[str, object]] = None
    """The properties of the path parameters."""

    required: Optional[List[str]] = None
    """The required properties of the path parameters."""

    type: Optional[Literal["object"]] = None


class QueryParameters(BaseModel):
    """The query parameters the webhook tool accepts, described as a JSON Schema object.

    These parameters will be passed to the webhook as the query of the request. See the [JSON Schema reference](https://json-schema.org/understanding-json-schema) for documentation about the format
    """

    properties: Optional[Dict[str, object]] = None
    """The properties of the query parameters."""

    required: Optional[List[str]] = None
    """The required properties of the query parameters."""

    type: Optional[Literal["object"]] = None


class InferenceEmbeddingWebhookToolParams(BaseModel):
    description: str
    """The description of the tool."""

    name: str
    """The name of the tool."""

    url: str
    """The URL of the external tool to be called.

    This URL is going to be used by the assistant. The URL can be templated like:
    `https://example.com/api/v1/{id}`, where `{id}` is a placeholder for a value
    that will be provided by the assistant if `path_parameters` are provided with
    the `id` attribute.
    """

    body_parameters: Optional[BodyParameters] = None
    """The body parameters the webhook tool accepts, described as a JSON Schema object.

    These parameters will be passed to the webhook as the body of the request. See
    the [JSON Schema reference](https://json-schema.org/understanding-json-schema)
    for documentation about the format
    """

    headers: Optional[List[Header]] = None
    """The headers to be sent to the external tool."""

    method: Optional[Literal["GET", "POST", "PUT", "DELETE", "PATCH"]] = None
    """The HTTP method to be used when calling the external tool."""

    path_parameters: Optional[PathParameters] = None
    """The path parameters the webhook tool accepts, described as a JSON Schema object.

    These parameters will be passed to the webhook as the path of the request if the
    URL contains a placeholder for a value. See the
    [JSON Schema reference](https://json-schema.org/understanding-json-schema) for
    documentation about the format
    """

    query_parameters: Optional[QueryParameters] = None
    """The query parameters the webhook tool accepts, described as a JSON Schema
    object.

    These parameters will be passed to the webhook as the query of the request. See
    the [JSON Schema reference](https://json-schema.org/understanding-json-schema)
    for documentation about the format
    """
